@@ -17,8 +17,8 @@ import os.path
 import textwrap
 import time
 
+import ddt
 import mock
-from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 
 from os_brick import exception
@@ -28,6 +28,7 @@ from os_brick.tests import base
 LOG = logging.getLogger(__name__)
 
 
+@ddt.ddt
 class LinuxSCSITestCase(base.TestCase):
     def setUp(self):
         super(LinuxSCSITestCase, self).setUp()
@@ -73,51 +74,51 @@ class LinuxSCSITestCase(base.TestCase):
             ('tee -a /sys/block/sdc/device/delete')]
         self.assertEqual(expected_commands, self.cmds)
 
-    @mock.patch('time.sleep')
-    def test_wait_for_volume_removal(self, sleep_mock):
-        fake_path = '/dev/disk/by-path/fake-iscsi-iqn-lun-0'
-        exists_mock = mock.Mock()
-        exists_mock.return_value = True
-        os.path.exists = exists_mock
-        self.assertRaises(exception.VolumePathNotRemoved,
-                          self.linuxscsi.wait_for_volume_removal,
-                          fake_path)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'echo_scsi_command')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_device_io')
+    @mock.patch.object(os.path, 'exists', return_value=True)
+    def test_remove_scsi_device_force(self, exists_mock, flush_mock,
+                                      echo_mock):
+        """With force we'll always call delete even if flush fails."""
+        exc = exception.ExceptionChainer()
+        flush_mock.side_effect = Exception()
+        echo_mock.side_effect = Exception()
+        device = '/dev/sdc'
 
-        exists_mock = mock.Mock()
-        exists_mock.return_value = False
-        os.path.exists = exists_mock
-        self.linuxscsi.wait_for_volume_removal(fake_path)
-        expected_commands = []
-        self.assertEqual(expected_commands, self.cmds)
-        self.assertTrue(sleep_mock.called)
+        self.linuxscsi.remove_scsi_device(device, force=True, exc=exc)
+        # The context manager has caught the exceptions
+        self.assertTrue(exc)
+        flush_mock.assert_called_once_with(device)
+        echo_mock.assert_called_once_with('/sys/block/sdc/device/delete', '1')
+
+    @mock.patch('time.sleep')
+    @mock.patch('os.path.exists', return_value=True)
+    def test_wait_for_volumes_removal_failure(self, exists_mock, sleep_mock):
+        retries = 3
+        names = ('sda', 'sdb')
+        self.assertRaises(exception.VolumePathNotRemoved,
+                          self.linuxscsi.wait_for_volumes_removal, names)
+        exists_mock.assert_has_calls([mock.call('/dev/' + name)
+                                      for name in names] * retries)
+        self.assertEqual(retries - 1, sleep_mock.call_count)
+
+    @mock.patch('time.sleep')
+    @mock.patch('os.path.exists', side_effect=(True, True, False, False))
+    def test_wait_for_volumes_removal_retry(self, exists_mock, sleep_mock):
+        names = ('sda', 'sdb')
+        self.linuxscsi.wait_for_volumes_removal(names)
+        exists_mock.assert_has_calls([mock.call('/dev/' + name)
+                                      for name in names] * 2)
+        self.assertEqual(1, sleep_mock.call_count)
 
     def test_flush_multipath_device(self):
-        self.linuxscsi.flush_multipath_device('/dev/dm-9')
-        expected_commands = [('multipath -f /dev/dm-9')]
-        self.assertEqual(expected_commands, self.cmds)
+        dm_map_name = '3600d0230000000000e13955cc3757800'
+        with mock.patch.object(self.linuxscsi, '_execute') as exec_mock:
+            self.linuxscsi.flush_multipath_device(dm_map_name)
 
-    @mock.patch('retrying.time.sleep', mock.Mock())
-    def test_flush_multipath_device_in_use(self):
-        side_effect = (
-            putils.ProcessExecutionError(
-                stdout='Feb 09 14:38:02 | mpatha: map in use\n'
-                       'Feb 09 14:38:02 | failed to remove multipath map '
-                       'mpatha\n',
-                exit_code=1),
-            ('', '')
-        )
-
-        with mock.patch.object(self.linuxscsi, '_execute') as execute_mock:
-            execute_mock.side_effect = side_effect
-            self.linuxscsi.flush_multipath_device('mpatha')
-            execute_mock.assert_has_calls(
-                [mock.call('multipath', '-f', 'mpatha', run_as_root=True,
-                           root_helper=mock.ANY)] * 2)
-
-    def test_flush_multipath_devices(self):
-        self.linuxscsi.flush_multipath_devices()
-        expected_commands = [('multipath -F')]
-        self.assertEqual(expected_commands, self.cmds)
+        exec_mock.assert_called_once_with(
+            'multipath', '-f', dm_map_name, run_as_root=True, attempts=3,
+            timeout=300, interval=10, root_helper=self.linuxscsi._root_helper)
 
     def test_get_scsi_wwn(self):
         fake_path = '/dev/disk/by-id/somepath'
@@ -130,6 +131,51 @@ class LinuxSCSITestCase(base.TestCase):
         wwn = self.linuxscsi.get_scsi_wwn(fake_path)
         self.assertEqual(fake_wwn, wwn)
 
+    @mock.patch('six.moves.builtins.open')
+    def test_get_dm_name(self, open_mock):
+        dm_map_name = '3600d0230000000000e13955cc3757800'
+        cm_open = open_mock.return_value.__enter__.return_value
+        cm_open.read.return_value = dm_map_name
+        res = self.linuxscsi.get_dm_name('dm-0')
+        self.assertEqual(dm_map_name, res)
+        open_mock.assert_called_once_with('/sys/block/dm-0/dm/name')
+
+    @mock.patch('six.moves.builtins.open', side_effect=IOError)
+    def test_get_dm_name_failure(self, open_mock):
+        self.assertEqual('', self.linuxscsi.get_dm_name('dm-0'))
+
+    @mock.patch('glob.glob', side_effect=[[], ['/sys/block/sda/holders/dm-9']])
+    def test_find_sysfs_multipath_dm(self, glob_mock):
+        device_names = ('sda', 'sdb')
+        res = self.linuxscsi.find_sysfs_multipath_dm(device_names)
+        self.assertEqual('dm-9', res)
+        glob_mock.assert_has_calls([mock.call('/sys/block/sda/holders/dm-*'),
+                                    mock.call('/sys/block/sdb/holders/dm-*')])
+
+    @mock.patch('glob.glob', return_value=[])
+    def test_find_sysfs_multipath_dm_not_found(self, glob_mock):
+        device_names = ('sda', 'sdb')
+        res = self.linuxscsi.find_sysfs_multipath_dm(device_names)
+        self.assertIsNone(res)
+        glob_mock.assert_has_calls([mock.call('/sys/block/sda/holders/dm-*'),
+                                    mock.call('/sys/block/sdb/holders/dm-*')])
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch('os.path.exists', return_value=True)
+    def test_flush_device_io(self, exists_mock, exec_mock):
+        device = '/dev/sda'
+        self.linuxscsi.flush_device_io(device)
+        exists_mock.assert_called_once_with(device)
+        exec_mock.assert_called_once_with(
+            'blockdev', '--flushbufs', device, run_as_root=True, attempts=3,
+            timeout=300, interval=10, root_helper=self.linuxscsi._root_helper)
+
+    @mock.patch('os.path.exists', return_value=False)
+    def test_flush_device_io_non_existent(self, exists_mock):
+        device = '/dev/sda'
+        self.linuxscsi.flush_device_io(device)
+        exists_mock.assert_called_once_with(device)
+
     @mock.patch.object(os.path, 'exists', return_value=True)
     def test_find_multipath_device_path(self, exists_mock):
         fake_wwn = '1234567890'
@@ -168,31 +214,81 @@ class LinuxSCSITestCase(base.TestCase):
                                 self.linuxscsi.wait_for_path,
                                 path)
 
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device')
-    @mock.patch.object(os.path, 'exists', return_value=True)
-    def test_remove_multipath_device(self, exists_mock, mock_multipath):
-        def fake_find_multipath_device(device):
-            devices = [{'device': '/dev/sde', 'host': 0,
-                        'channel': 0, 'id': 0, 'lun': 1},
-                       {'device': '/dev/sdf', 'host': 2,
-                        'channel': 0, 'id': 0, 'lun': 1}, ]
+    @ddt.data({'do_raise': False, 'force': False},
+              {'do_raise': True, 'force': True})
+    @ddt.unpack
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    def test_remove_connection_multipath_complete(self, remove_mock, wait_mock,
+                                                  find_dm_mock,
+                                                  get_dm_name_mock,
+                                                  flush_mp_mock,
+                                                  remove_link_mock,
+                                                  do_raise, force):
+        if do_raise:
+            flush_mp_mock.side_effect = Exception
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+        mp_name = self.linuxscsi.remove_connection(devices_names,
+                                                   is_multipath=True,
+                                                   force=mock.sentinel.Force,
+                                                   exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        get_dm_name_mock.assert_called_once_with(find_dm_mock.return_value)
+        flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
+        self.assertEqual(get_dm_name_mock.return_value if do_raise else None,
+                         mp_name)
+        remove_mock.assert_has_calls([
+            mock.call('/dev/sda', mock.sentinel.Force, exc),
+            mock.call('/dev/sdb', mock.sentinel.Force, exc)])
+        wait_mock.assert_called_once_with(devices_names)
+        self.assertEqual(do_raise, bool(exc))
+        remove_link_mock.assert_called_once_with(devices_names)
 
-            info = {"device": "dm-3",
-                    "id": "350002ac20398383d",
-                    "name": "mpatha",
-                    "devices": devices}
-            return info
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device',
+                       side_effect=Exception)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    def test_remove_connection_multipath_fail(self, remove_mock, wait_mock,
+                                              find_dm_mock, get_dm_name_mock,
+                                              flush_mp_mock, remove_link_mock):
+        flush_mp_mock.side_effect = exception.ExceptionChainer
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+        self.assertRaises(exception.ExceptionChainer,
+                          self.linuxscsi.remove_connection,
+                          devices_names, is_multipath=True,
+                          force=False, exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        get_dm_name_mock.assert_called_once_with(find_dm_mock.return_value)
+        flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
+        remove_mock.assert_not_called()
+        wait_mock.assert_not_called()
+        remove_link_mock.assert_not_called()
+        self.assertTrue(bool(exc))
 
-        mock_multipath.side_effect = fake_find_multipath_device
-
-        self.linuxscsi.remove_multipath_device('/dev/dm-3')
-        expected_commands = [
-            ('multipath -f mpatha'),
-            ('blockdev --flushbufs /dev/sde'),
-            ('tee -a /sys/block/sde/device/delete'),
-            ('blockdev --flushbufs /dev/sdf'),
-            ('tee -a /sys/block/sdf/device/delete'), ]
-        self.assertEqual(expected_commands, self.cmds)
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    def test_remove_connection_singlepath(self, remove_mock, wait_mock,
+                                          remove_link_mock):
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+        self.linuxscsi.remove_connection(devices_names, is_multipath=False,
+                                         force=mock.sentinel.Force,
+                                         exc=exc)
+        remove_mock.assert_has_calls(
+            [mock.call('/dev/sda', mock.sentinel.Force, exc),
+             mock.call('/dev/sdb', mock.sentinel.Force, exc)])
+        wait_mock.assert_called_once_with(devices_names)
+        remove_link_mock.assert_called_once_with(devices_names)
 
     def test_find_multipath_device_3par_ufn(self):
         def fake_execute(*cmd, **kwargs):
@@ -671,3 +767,189 @@ loop0                                     0"""
                 False, None, mock_rootwrap.execute))
         mock_rootwrap.execute.assert_called_once_with(
             'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
+
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_single_designator(self, get_wwid_mock, glob_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-wwid2']
+        get_wwid_mock.return_value = 'wwid1'
+        res = self.linuxscsi.get_sysfs_wwn(mock.sentinel.device_names)
+        self.assertEqual('wwid1', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
+
+    @mock.patch('os.path.realpath', side_effect=('/other/path',
+                                                 '/dev/sda', '/dev/sdb'))
+    @mock.patch('os.path.islink', side_effect=(False, True, True, True, True))
+    @mock.patch('os.stat', side_effect=(False, True, True, True))
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_multiple_designators(self, get_wwid_mock, glob_mock,
+                                                stat_mock, islink_mock,
+                                                realpath_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-fail-link',
+                                  '/dev/disk/by-id/scsi-fail-stat',
+                                  '/dev/disk/by-id/scsi-non-dev',
+                                  '/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-wwid2']
+        get_wwid_mock.return_value = 'pre-wwid'
+        devices = ['sdb', 'sdc']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('wwid2', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(devices)
+
+    @mock.patch('os.path.realpath', side_effect=('/dev/sda', '/dev/sdb'))
+    @mock.patch('os.path.islink', return_value=True)
+    @mock.patch('os.stat', return_value=True)
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_not_found(self, get_wwid_mock, glob_mock, stat_mock,
+                                     islink_mock, realpath_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-wwid2']
+        get_wwid_mock.return_value = 'pre-wwid'
+        devices = ['sdc']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(devices)
+
+    @ddt.data({'wwn_type': 't10.', 'num_val': '1'},
+              {'wwn_type': 'eui.', 'num_val': '2'},
+              {'wwn_type': 'naa.', 'num_val': '3'})
+    @ddt.unpack
+    @mock.patch('six.moves.builtins.open')
+    def test_get_sysfs_wwid(self, open_mock, wwn_type, num_val):
+        read_fail = mock.MagicMock()
+        read_fail.__enter__.return_value.read.side_effect = IOError
+        read_data = mock.MagicMock()
+        read_data.__enter__.return_value.read.return_value = (wwn_type +
+                                                              'wwid1\n')
+        open_mock.side_effect = (IOError, read_fail, read_data)
+
+        res = self.linuxscsi.get_sysfs_wwid(['sda', 'sdb', 'sdc'])
+        self.assertEqual(num_val + 'wwid1', res)
+        open_mock.assert_has_calls([mock.call('/sys/block/sda/device/wwid'),
+                                    mock.call('/sys/block/sdb/device/wwid'),
+                                    mock.call('/sys/block/sdc/device/wwid')])
+
+    @mock.patch('six.moves.builtins.open', side_effect=IOError)
+    def test_get_sysfs_wwid_not_found(self, open_mock):
+        res = self.linuxscsi.get_sysfs_wwid(['sda', 'sdb'])
+        self.assertEqual('', res)
+        open_mock.assert_has_calls([mock.call('/sys/block/sda/device/wwid'),
+                                    mock.call('/sys/block/sdb/device/wwid')])
+
+    @mock.patch.object(linuxscsi.priv_rootwrap, 'unlink_root')
+    @mock.patch('glob.glob')
+    @mock.patch('os.path.realpath', side_effect=['/dev/sda', '/dev/sdb',
+                                                 '/dev/sdc'])
+    def test_remove_scsi_symlinks(self, realpath_mock, glob_mock, unlink_mock):
+        paths = ['/dev/disk/by-id/scsi-wwid1', '/dev/disk/by-id/scsi-wwid2',
+                 '/dev/disk/by-id/scsi-wwid3']
+        glob_mock.return_value = paths
+        self.linuxscsi._remove_scsi_symlinks(['sdb', 'sdc', 'sdd'])
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        realpath_mock.assert_has_calls([mock.call(g) for g in paths])
+        unlink_mock.assert_called_once_with(no_errors=True, *paths[1:])
+
+    @mock.patch.object(linuxscsi.priv_rootwrap, 'unlink_root')
+    @mock.patch('glob.glob')
+    @mock.patch('os.path.realpath', side_effect=['/dev/sda', '/dev/sdb'])
+    def test_remove_scsi_symlinks_no_links(self, realpath_mock, glob_mock,
+                                           unlink_mock):
+        paths = ['/dev/disk/by-id/scsi-wwid1', '/dev/disk/by-id/scsi-wwid2']
+        glob_mock.return_value = paths
+        self.linuxscsi._remove_scsi_symlinks(['/dev/sdd', '/dev/sde'])
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        realpath_mock.assert_has_calls([mock.call(g) for g in paths])
+        unlink_mock.assert_not_called()
+
+    @mock.patch.object(linuxscsi.priv_rootwrap, 'unlink_root')
+    @mock.patch('glob.glob')
+    @mock.patch('os.path.realpath', side_effect=[OSError, '/dev/sda'])
+    def test_remove_scsi_symlinks_race_condition(self, realpath_mock,
+                                                 glob_mock, unlink_mock):
+        paths = ['/dev/disk/by-id/scsi-wwid1', '/dev/disk/by-id/scsi-wwid2']
+        glob_mock.return_value = paths
+        self.linuxscsi._remove_scsi_symlinks(['sda'])
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        realpath_mock.assert_has_calls([mock.call(g) for g in paths])
+        unlink_mock.assert_called_once_with(paths[1], no_errors=True)
+
+    @mock.patch('glob.glob')
+    def test_get_hctl_with_target(self, glob_mock):
+        glob_mock.return_value = [
+            '/sys/class/iscsi_host/host3/device/session1/target3:4:5',
+            '/sys/class/iscsi_host/host3/device/session1/target3:4:6']
+        res = self.linuxscsi.get_hctl('1', '2')
+        self.assertEqual(('3', '4', '5', '2'), res)
+        glob_mock.assert_called_once_with(
+            '/sys/class/iscsi_host/host*/device/session1/target*')
+
+    @mock.patch('glob.glob')
+    def test_get_hctl_no_target(self, glob_mock):
+        glob_mock.side_effect = [
+            [],
+            ['/sys/class/iscsi_host/host3/device/session1',
+             '/sys/class/iscsi_host/host3/device/session1']]
+        res = self.linuxscsi.get_hctl('1', '2')
+        self.assertEqual(('3', '-', '-', '2'), res)
+        glob_mock.assert_has_calls(
+            [mock.call('/sys/class/iscsi_host/host*/device/session1/target*'),
+             mock.call('/sys/class/iscsi_host/host*/device/session1')])
+
+    @mock.patch('glob.glob', return_value=[])
+    def test_get_hctl_no_paths(self, glob_mock):
+        res = self.linuxscsi.get_hctl('1', '2')
+        self.assertIsNone(res)
+        glob_mock.assert_has_calls(
+            [mock.call('/sys/class/iscsi_host/host*/device/session1/target*'),
+             mock.call('/sys/class/iscsi_host/host*/device/session1')])
+
+    @mock.patch('glob.glob')
+    def test_device_name_by_hctl(self, glob_mock):
+        glob_mock.return_value = [
+            '/sys/class/scsi_host/host3/device/session1/target3:4:5/3:4:5:2/'
+            'block/sda2',
+            '/sys/class/scsi_host/host3/device/session1/target3:4:5/3:4:5:2/'
+            'block/sda']
+        res = self.linuxscsi.device_name_by_hctl('1', ('3', '4', '5', '2'))
+        self.assertEqual('sda', res)
+        glob_mock.assert_called_once_with(
+            '/sys/class/scsi_host/host3/device/session1/target3:4:5/3:4:5:2/'
+            'block/*')
+
+    @mock.patch('glob.glob')
+    def test_device_name_by_hctl_wildcards(self, glob_mock):
+        glob_mock.return_value = [
+            '/sys/class/scsi_host/host3/device/session1/target3:4:5/3:4:5:2/'
+            'block/sda2',
+            '/sys/class/scsi_host/host3/device/session1/target3:4:5/3:4:5:2/'
+            'block/sda']
+        res = self.linuxscsi.device_name_by_hctl('1', ('3', '-', '-', '2'))
+        self.assertEqual('sda', res)
+        glob_mock.assert_called_once_with(
+            '/sys/class/scsi_host/host3/device/session1/target3:*:*/3:*:*:2/'
+            'block/*')
+
+    @mock.patch('glob.glob', mock.Mock(return_value=[]))
+    def test_device_name_by_hctl_no_devices(self):
+        res = self.linuxscsi.device_name_by_hctl('1', ('4', '5', '6', '2'))
+        self.assertIsNone(res)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'echo_scsi_command')
+    def test_scsi_iscsi(self, echo_mock):
+        self.linuxscsi.scan_iscsi('host', 'channel', 'target', 'lun')
+        echo_mock.assert_called_once_with('/sys/class/scsi_host/hosthost/scan',
+                                          'channel target lun')
+
+    def test_multipath_add_wwid(self):
+        self.linuxscsi.multipath_add_wwid('wwid1')
+        self.assertEqual(['multipath -a wwid1'], self.cmds)
+
+    def test_multipath_add_path(self):
+        self.linuxscsi.multipath_add_path('/dev/sda')
+        self.assertEqual(['multipathd add path /dev/sda'], self.cmds)
